Control kernel analysis code
----------------------------

2020-10-30

The code in this directory can be used to recreate the results in the manuscript "The basis of easy controllability in Boolean networks" by Enrico Borriello and Bryan C. Daniels.


## Dependencies

- Python 3

- Neet, development version (`modularity` branch, commit `87e70e0`)  
Available here: https://github.com/bcdaniels/Neet

- Datadex, development version (`dev` branch, commit `f4c2070`)  
Available here: https://github.com/ELIFE-ASU/datadex

The last two dependencies can be installed by cloning each public git repository (e.g. `git clone https://github.com/bcdaniels/Neet`), switching to the appropriate git branch (e.g. `cd Neet`, `git checkout modularity`), and installing using pip (e.g. `pip install -e ./`).

Given a typical research Python 3 environment, downloading and installing these dependencies should take only a few minutes on a standard desktop computer.


## Example control kernel analysis

To run the exact control kernel analysis for the network with index `14` in the included Cell Collective database (corresponding to the network named `Cell Cycle Transcription By Coupled CDK And Network Oscillators`), run

```
python run_control_kernels.py 14
```

This will output control kernel data as a dictionary to a pickled file named `control_kernel_{Name of network}.dat`.  

To run the sampling analysis on the same network, run 

```
python run_sampled_control_kernels.py 14
```

This will output control kernel data as a dictionary to a pickled file named `sampled_control_kernel_{Name of network}.dat`.

Running this example analysis should take a few seconds on a standard desktop computer.

To view the corresponding data, load the pickled files in phython.  For example, the following python code prints to the screen the attractors and control kernels of the example network:

```
from toolbox.simplePickle import load
data = load("control_kernel_Cell Cycle Transcription By Coupled CDK And Network Oscillators.dat")
for att,ck in zip(data['attractors'],data['control_kernels']):
	if ck:
		print("Attractor {} has control kernel {}.".format(att,ck))
	else:
		print("Attractor {} does not have a static control kernel.".format(att))
```  

The output dictionaries include data on attractors, corresponding control kernels, distinguishing nodes, and the iterative bound process.

To run a similar analysis on a random Boolean network, use the file `run_rbn_control_kernels.py`.

## Random vector witness set analysis

The file `avg_delta.py` calculates witness set sizes for randomly generated binary vectors of fixed length `n` and varying number `r`. Results are saved to a CSV file named, for instance, `avg_delta_data_n20.csv`.


## How to run the software on your data

The exact and sampled control kernel and distinguishing node analyses can be performed on an arbitrary Boolean Neet network using the functions `ck_analysis`, `dn_analysis`, `sampled_ck_analysis`, and `sampled_dn_analysis` in the file `control_kernel_analysis.py`.
