# avg_delta.py
#
# From Enrico Borriello
# 2020/06/19
#
# Calculate average delta values (average number of distinguishing nodes)
# for sets of random binary numbers.
#

import numpy as np
import itertools
import matplotlib.pyplot as plt
import random
import pandas as pd

def distinguishing_nodes(attractors,att_index):

    selected_attractor = attractors[att_index]
    remaining_attractors =  attractors[:att_index] + attractors[att_index+1 :]

    num_diff = 0
    comb_len = 0
    while num_diff != len(remaining_attractors):
        comb_len = comb_len +1
        comb = list(itertools.combinations(range(len(attractors[0])),comb_len))
        possibilities = len(list(comb))
        comb_index = 0
        while num_diff != len(remaining_attractors) and comb_index < possibilities:
            selected_nodes_values = [selected_attractor[i] for i in list(comb)[comb_index]]
            num_diff = 0
            for other_attractor in remaining_attractors:
                other_nodes_values = [other_attractor[i] for i in list(comb)[comb_index]]
                num_diff = num_diff + int(selected_nodes_values != other_nodes_values)
            if num_diff == len(remaining_attractors):
                return comb[comb_index]
            else:
                comb_index = comb_index + 1

def avg_delta(attractors):
    delta = []
    for i in range(len(attractors)):
        delta_i = len(distinguishing_nodes(attractors,i))
        delta.append(delta_i)
    return np.mean(delta)

def distribution_parameters_fixed_r (r,sim=100):
    avgd_dist = []
    for _ in range(sim):
        attractors = random.sample(tuples,r)
        avgd = avg_delta(attractors)
        avgd_dist.append(avgd)
    return [r,np.mean(avgd_dist), np.std(avgd_dist), min(avgd_dist), max(avgd_dist)]


n = 20  # size of the network

tuples = list(itertools.product(*[(0, 1)] * n))

rmin = 2
rmax = 2**n
bins = 10
start = np.log10(rmin)
scale = float(np.log10(rmax)-np.log10(rmin))/bins

sim = 10  # number of time we sample 

data = []
for i in range(bins+1):
    r = int(10**(start+scale*i))
    data.append(distribution_parameters_fixed_r (r,sim))
    print r
    
    df = pd.DataFrame(data)

    df.to_csv('avg_delta_data_n{}.csv'.format(n),header=False)










