# run_sampled_control_kernels.py
#
# Bryan Daniels
# 2.19.2019
# 5.21.2019 branched from run_control_kernels.py
#
# Find control kernels for each attractor for one network in the
# Cell Collective database.  The network's index is set by command
# line argument.
#

import sys
import datadex
from toolbox.simplePickle import save
from load_cell_collective_nets import loadCellCollectiveNets
from control_kernel_analysis import sampled_ck_analysis,sampled_dn_analysis

if len(sys.argv) != 2:
    print("Usage: python run_sampled_control_kernels.py [netIndex]")
    exit()

# () set run parameters

sampleseed = 130
samples = 10000 #10000 #1000
phenotype = 'all' # 'internal'
phenotype_only = False # False (runs on and before 8/12/19 have effective value False)
require_inputs = True # True # False (runs before 5/1/20 have effective value False)

# () load cell collective network as net

netDict = loadCellCollectiveNets( netindex = int(sys.argv[1]) )
netname = list(netDict.keys())[0]
net = netDict[netname]
print("Loaded network: {}".format(netname))
print()

# () run sampled distinguishing node analysis

data = sampled_dn_analysis(net,seed=sampleseed,samples=samples,
                           phenotype=phenotype,require_inputs=require_inputs)

# () save output data to pickled file

data["name"] = netname
#data["net"] = net
filename = "sampled_control_kernel_{}.dat".format(netname)
print("Outputting distinguishing node data to {}".format(filename))
print()
save(data,filename)

# () run sampled control kernel analysis

data_ck = sampled_ck_analysis(net,seed=sampleseed,samples=samples,
                           phenotype=phenotype,phenotype_only=phenotype_only,
                           require_inputs=require_inputs,iterative=False)

# () save output data to pickled file

data.update(data_ck)
data["name"] = netname
#data["net"] = net
filename = "sampled_control_kernel_{}.dat".format(netname)
print("Outputting control kernel data to {}".format(filename))
print()
save(data,filename)

# () run sampled control kernel analysis (iterative)

data_ck = sampled_ck_analysis(net,seed=sampleseed,samples=samples,
                           phenotype=phenotype,phenotype_only=phenotype_only,
                           require_inputs=require_inputs,iterative=True)

# () save output data to pickled file

data.update(data_ck)
data["name"] = netname
#data["net"] = net
filename = "sampled_control_kernel_{}.dat".format(netname)
print("Outputting iterative control kernel data to {}".format(filename))
print()
save(data,filename)
